// Generated by CoffeeScript 1.9.1
(function() {
  var Helpers, W, _, fs, glob, nodefn, path, rimraf, run;

  require('colors');

  fs = require('fs');

  path = require('path');

  glob = require('glob');

  rimraf = require('rimraf');

  W = require('when');

  nodefn = require('when/node');

  run = require('child_process').exec;

  _ = require('lodash');


  /**
   * @class Helpers
   * @classdesc a collection of useful test helper functions for
   * roots extensions
   */

  Helpers = (function() {
    function Helpers(opts) {
      var _asyncReadFile, _exists, _path;
      if (opts == null) {
        opts = {};
      }
      _path = function(f) {
        if (opts.base != null) {
          return path.join(opts.base, f);
        } else {
          return path.normalize(f);
        }
      };
      _exists = function(file, opts) {
        if (opts == null) {
          opts = {
            async: false
          };
        }
        if (opts.async) {
          return nodefn.call(fs.stat, file).then(function() {
            return true;
          })["catch"](function() {
            return false;
          });
        } else {
          try {
            return fs.statSync(file) != null;
          } catch (_error) {
            return false;
          }
        }
      };
      _asyncReadFile = function(file) {
        return nodefn.call(fs.readFile, file, 'utf8');
      };
      this.file = {
        exists: function(f, opts) {
          return _exists(_path(f), opts);
        },
        doesnt_exist: function(f, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _exists(_path(f), opts).then(function(exists) {
              return !exists;
            });
          } else {
            return !_exists(_path(f));
          }
        },
        has_content: function(f, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _asyncReadFile(_path(f)).then(function(c) {
              return c.length > 0;
            });
          } else {
            return fs.readFileSync(_path(f), 'utf8').length > 0;
          }
        },
        is_empty: function(f, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _asyncReadFile(_path(f)).then(function(c) {
              return c.length < 1;
            });
          } else {
            return fs.readFileSync(_path(f), 'utf8').length < 1;
          }
        },
        contains: function(f, content, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _asyncReadFile(_path(f)).then(function(c) {
              return c.indexOf(content) > -1;
            });
          } else {
            return fs.readFileSync(_path(f), 'utf8').indexOf(content) > -1;
          }
        },
        contains_match: function(f, regex, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _asyncReadFile(_path(f)).then(function(c) {
              return !!c.match(regex);
            });
          } else {
            return !!fs.readFileSync(_path(f), 'utf8').match(regex);
          }
        },
        matches_file: function(f, expected, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          f = _path(f);
          expected = _path(expected);
          if (opts.async) {
            return W.all([_asyncReadFile(f), _asyncReadFile(expected)]).spread(function(f, expected) {
              return f === expected;
            });
          } else {
            return String(fs.readFileSync(f)) === String(fs.readFileSync(expected));
          }
        }
      };
      this.directory = {
        is_directory: function(dir, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return nodefn.call(fs.stat, _path(dir)).then(function(s) {
              return s.isDirectory();
            });
          } else {
            return fs.statSync(_path(dir)).isDirectory();
          }
        },
        exists: function(dir, opts) {
          var stat;
          if (opts == null) {
            opts = {
              async: false
            };
          }
          dir = _path(dir);
          if (opts.async) {
            return nodefn.call(fs.stat, dir).then(function(s) {
              return s.isDirectory();
            })["catch"](function() {
              return false;
            });
          } else {
            try {
              stat = fs.statSync(dir);
            } catch (_error) {
              return false;
            }
            return stat.isDirectory();
          }
        },
        doesnt_exist: function(dir, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return _exists(_path(dir), opts).then(function(exists) {
              return !exists;
            });
          } else {
            return !_exists(_path(dir));
          }
        },
        has_contents: function(dir, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return nodefn.call(fs.readdir, _path(dir)).then(function(d) {
              return d.length > 0;
            });
          } else {
            return fs.readdirSync(_path(dir)).length > 0;
          }
        },
        is_empty: function(dir, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return nodefn.call(fs.readdir, _path(dir)).then(function(d) {
              return d.length < 1;
            });
          } else {
            return fs.readdirSync(_path(dir)).length < 1;
          }
        },
        contains_file: function(dir, file, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return nodefn.call(fs.readdir, _path(dir)).then(function(d) {
              return _.contains(d, file);
            });
          } else {
            return _.contains(fs.readdirSync(_path(dir)), file);
          }
        },
        matches_dir: function(dir, expected, opts) {
          if (opts == null) {
            opts = {
              async: false
            };
          }
          dir = _path(dir);
          expected = _path(expected);
          if (opts.async) {
            return W.all([nodefn.call(fs.readdir, dir), nodefn.call(fs.readdir, expected)]).spread(function(dir, expected) {
              return String(dir) === String(expected);
            });
          } else {
            return String(fs.readdirSync(dir)) === String(fs.readdirSync(expected));
          }
        }
      };
      this.project = {
        compile: function(Roots, p) {
          var proj;
          Roots.analytics({
            disable: true
          });
          proj = new Roots(_path(p));
          proj.on('error', function() {});
          return proj.compile().then(function() {
            return Roots.analytics({
              enable: true
            });
          });
        },
        remove_folders: function(matcher, opts) {
          var dir, i, len, ref, results;
          if (opts == null) {
            opts = {
              async: false
            };
          }
          if (opts.async) {
            return W.map(nodefn.call(glob, _path(matcher)), function(dir) {
              return nodefn.call(rimraf, dir);
            });
          } else {
            ref = glob.sync(_path(matcher));
            results = [];
            for (i = 0, len = ref.length; i < len; i++) {
              dir = ref[i];
              results.push(rimraf.sync(dir));
            }
            return results;
          }
        },
        install_dependencies: function(base, cb) {
          var dirs;
          dirs = nodefn.call(glob, (_path(base)) + "/package.json");
          return W.map(dirs, function(d) {
            return path.dirname(d);
          }).then(function(dirs) {
            return W.filter(dirs, function(d) {
              return !_exists(path.join(d, 'node_modules'));
            });
          }).tap(function(tasks) {
            if (tasks.length) {
              console.log('installing test dependencies...'.grey);
            }
            return W.map(tasks, function(d) {
              return nodefn.call(run, "npm i", {
                cwd: d
              });
            });
          }).then(cb);
        }
      };
    }

    return Helpers;

  })();

  module.exports = Helpers;

}).call(this);
